# This script file provides fundamental computational functionality for testing
# the time complexity of Python algorithms that focus on the computation of 
# the Fibonacci series.

# Import required Python libraries.
import time


# -----------------------------------------------------------------------------
#                        FUNCTIONS DEFINITION SEGMENT:
# -----------------------------------------------------------------------------
#                          Tradional Approach:
# -----------------------------------------------------------------------------
def fib1(n):
    fibs = [0,1]
    for i in range(2,n+1):
        fibs.append(fibs[-1]+fibs[-2])
    return fibs
# -----------------------------------------------------------------------------
#                         Memoization Approach:
# -----------------------------------------------------------------------------
def fib2(n,computed={0:0,1:1}):
    if n not in computed:
        computed[n] = fib2(n-1,computed) + fib2(n-2,computed)
    return computed[n]
# -----------------------------------------------------------------------------
#                         Naive Iterative Approach:
# -----------------------------------------------------------------------------
def fib3(n):
    a,b = 0,1
    for _ in range(n):
        a,b = b,a+b
    return a
# -----------------------------------------------------------------------------
#                        MAIN CODE SEGMENT:
# -----------------------------------------------------------------------------

# Set the number of the Fibonacci sequence up to which to perform the 
# computation. 
N = 800

# Test the computation speed of the first function.
start_time = time.time() # Get the current time.
F1 = fib1(N)
dt1 = time.time() - start_time # Get the time duration.
print(f"F = {F1}")
# Test the computation speed of the second function.
start_time = time.time() # Get the current time.
F2 = [fib2(x) for x in range(0,N+1)]
dt2 = time.time() - start_time # Get the time duration.
print(f"F = {F2}")
# Test the computation speed of the third function.
start_time = time.time() # Get the current time.
F3 = [fib3(x) for x in range(0,N+1)]
dt3 = time.time() - start_time # Get the time duration.
print(f"F = {F3}")
# Report computation times.
print(f"Function fib1() computed F in {dt1} seconds")
print(f"Function fib2() computed F in {dt2} seconds")
print(f"Function fib3() computed F in {dt3} seconds")

# Test the execution time of the last two function when computing the last 
# element of the sequence.
start_time = time.time() # Get the current time.
fib2(N)
dta = time.time() - start_time # Get the time duration.
print(f"Function fib2({N}) executed in {dta} seconds")
start_time = time.time() # Get the current time.
fib3(N)
dtb = time.time() - start_time # Get the time duration.
print(f"Function fib3({N}) executed in {dtb} seconds")